#include "JSVCatBot.h"

//edit 12-09-2018:
//switched Pin1 and Pin2 of the motorcircuits because the use of the tone function interferes with PWM on pin 11 (see arduino forums)
//By using Pin11 just as a digital output this interference is avoided, in the code only Pin1 recieves the analogWrite instruction

//edit 20-09-2018
//switched pins back again because pin 10 and 9 cant be used in conjunction with the servo.h library

JSVCatBot::JSVCatBot(){
	//setting pins
	pinMode(MotorLeftPin1,OUTPUT);
	pinMode(MotorLeftPin2,OUTPUT);
	pinMode(MotorRightPin1,OUTPUT);
	pinMode(MotorRightPin2,OUTPUT);
	LeftMotor = JSVMotor(MotorLeftPin1,MotorLeftPin2,MinMotorSpeed,MaxMotorSpeed);
	RightMotor = JSVMotor(MotorRightPin1,MotorRightPin2,MinMotorSpeed,MaxMotorSpeed);
	pinMode(TrigPin,OUTPUT);
	pinMode(EchoPin,INPUT);
	//pinMode(LeftLEDPin,OUTPUT);
	//pinMode(RightLEDPin,OUTPUT);
	//pinMode(SpeakerPin,OUTPUT);
	//CANNOT INITIALISE SERVO HERE DUE TO POINTERS AND TIMERS AND SO ON.... (SEE FORUMS) trust me I spent 2 hours figuring this out!
	//WheelServo.attach(WheelServoServoPin);
	//HeadServo.attach(HeadServoServoPin);
  	Sonar = NewPing(TrigPin,EchoPin,MaxDistanceCm);
}

//MOTORS
void JSVCatBot::control_motor(int index, bool MotorDirection, uint8_t Percentage){
  if (Percentage > 100){
    Percentage = 100;
  }
  if (index==0){
	LeftMotor.ControlMotor(MotorDirection,Percentage);
  }
  if (index==1){
  RightMotor.ControlMotor(MotorDirection,Percentage);
  }
}
void JSVCatBot::CatSpeed(bool MotorDirection, uint8_t Percentage){
  control_motor(0,MotorDirection,Percentage);
  control_motor(1,MotorDirection,Percentage);
}
void JSVCatBot::SetupLeftMotor(bool ReverseMotor, int Percentage){
 	if (Percentage > 100){
    	Percentage = 100;
  	}
  	if (ReverseMotor){
		LeftMotor.ReverseMotor();	
	}
	LeftMotor.SetMaxMotorSpeed(MaxMotorSpeed*Percentage/100);
  	LeftMotor.SetMinMotorSpeed(MinMotorSpeed*Percentage/100);
}

void JSVCatBot::SetupRightMotor(bool ReverseMotor, int Percentage){
 	if (Percentage > 100){
    	Percentage = 100;
  	}
  	if (ReverseMotor){
		RightMotor.ReverseMotor();	
	}
	RightMotor.SetMaxMotorSpeed(MaxMotorSpeed*Percentage/100);
  	RightMotor.SetMinMotorSpeed(MinMotorSpeed*Percentage/100);
}

//ULTRASONIC SENSOR
unsigned int JSVCatBot::getDistance(){
    return Sonar.convert_cm(Sonar.ping_median(PingQuality));
}

//LED
/*
void JSVCatBot::LeftLEDOn(){
	digitalWrite(LeftLEDPin,HIGH);
}
void JSVCatBot::LeftLEDOff(){
	digitalWrite(LeftLEDPin,LOW);
}
void JSVCatBot::RightLEDOn(){
	digitalWrite(RightLEDPin,HIGH);
}
void JSVCatBot::RightLEDOff(){
	digitalWrite(RightLEDPin,LOW);
}
void JSVCatBot::LeftLEDSet(bool set){
	if (set) LeftLEDOn();
	else LeftLEDOff();
}
void JSVCatBot::RightLEDSet(bool set){
	if (set) RightLEDOn();
	else RightLEDOff();
}
void JSVCatBot::LeftLEDToggle(){
	if (!digitalRead(LeftLEDPin)) LeftLEDOn();
	else LeftLEDOff();
}
void JSVCatBot::RightLEDToggle(){
	if (!digitalRead(RightLEDPin)) RightLEDOn();
	else RightLEDOff();
}

//SPEAKER

void JSVCatBot::SpeakerPlayNote(int frequency){
	tone(SpeakerPin,frequency);
}
void JSVCatBot::SpeakerPlayNoteTime(int frequency, int time_ms){
	tone(SpeakerPin,frequency,time_ms);
	delay(time_ms);
}
void JSVCatBot::SpeakerStop(){
	noTone(SpeakerPin);
}
*/
//BLUETOOTH

void JSVCatBot::BluetoothOpenConnection(){
	Serial.begin(BluetoothBaudRate);
}
bool JSVCatBot::BluetoothHasData(){
	return Serial.available();
}
char JSVCatBot::BluetoothGetData(){
	return Serial.read();
}
void JSVCatBot::BluetoothSendData(String data){
	for (int i = 0; i < data.length(); i++)
  {
    Serial.write(data[i]);   // Push each char 1 by 1 on each loop pass
  }
}
void JSVCatBot::BluetoothSendData(double data){
	Serial.println(data);
}

//Servo
void JSVCatBot::InitServos(){
  WheelServo.attach(WheelServoServoPin);
  HeadServo.attach(HeadServoServoPin);
}

void JSVCatBot::SetServoAngle(Servo servomotor, int angle){
	servomotor.write(angle);
}
void JSVCatBot::SetWheelServoAngle(int angle){
	if (angle > 50){
		angle = 50;
	}
	if (angle < -50){
		angle = -50;
	}
	angle = WheelServoCenterAngle + angle;
	SetServoAngle(WheelServo,angle);
}
void JSVCatBot::SetHeadServoAngle(int angle){
		if (angle > 50){
		angle = 50;
	}
	if (angle < -50){
		angle = -50;
	}
	angle = HeadServoCenterAngle - angle;
	SetServoAngle(HeadServo,angle);
}
int JSVCatBot::ReadServoAngle(Servo servomotor){
  return servomotor.read();
}
int JSVCatBot::ReadWheelServoAngle(){
	return ReadServoAngle(WheelServo);
}
int JSVCatBot::ReadHeadServoAngle(){
	return ReadServoAngle(HeadServo);
}
